<!--- 
File:			Cart.cfm	
Description:	Manages shopping cart session array
Author:			Barry Moore	
Created:		26-Apr-2002
--->

<!--- Cart Architecture
		We will use a two dimensional array to keep keep track
		of items in the cart. We will store the array in a Session 
		scope variable so that it is avaliable for the duration of 
		the customer's session
		
		Each row in the array will hold four elements as outlined below
		1 - ProductNo
		2 - ProductName
		3 - Quantity
		4 - Unit Price

		Don't forget all session scope variables should be locked before use
 --->
 
 <!--- check to see if the array exists, if not create it --->
<CFIF NOT IsDefined("SESSION.aCart")>
		<CFLOCK TIMEOUT="10" THROWONTIMEOUT="No" TYPE="EXCLUSIVE" SCOPE="SESSION">
			<CFSET SESSION.aCart=ArrayNew(2)>
		</CFLOCK>
</CFIF>

<!--- set a default URL.Action variable if none exists  --->
<CFPARAM NAME="URL.Action" DEFAULT="Display">

<HTML>
  <HEAD>
      <TITLE>Beelze-Bubba Shopping Cart</TITLE>
	  <LINK REL="stylesheet" HREF="../styles/bbstyle.css" TYPE="text/css">
  </HEAD>
  
  <BODY BGCOLOR="#FFFFFF" TEXT="#000000" LEFTMARGIN="0" TOPMARGIN="0" MARGINWIDTH="0" MARGINHEIGHT="0" onLoad="self.focus()">
    <!--- header table --->
	<TABLE WIDTH="450" ALIGN="Center" BGCOLOR="#CC0000" CELLSPACING="0" CELLPADDING="3">
   		<TR>
   			<TD CLASS="Date"> Beelze-Bubba Shopping Cart </TD>
   		</TR>
   	</TABLE>
	<BR>
	<!--- use the URL.Action variable to determine what action to perform --->
	<CFSWITCH EXPRESSION="#URL.Action#">
	
		<!--- display the contents of the cart --->
		<CFCASE VALUE="Display">
		<!--- check to see if there are any items in the cart  --->
		<CFLOCK TIMEOUT="10" THROWONTIMEOUT="No" TYPE="READONLY" SCOPE="SESSION">
		  <CFIF ArrayLen(SESSION.aCart) GT 0>
			<TABLE WIDTH="450" BORDER="0" CELLSPACING="0" CELLPADDING="3" ALIGN="Center">
				<TR>
					<TD COLSPAN="5">
						The current contents of your shopping cart are listed below. 
						You can use the Change Quantities button to amend the quantity of all items.
					</TD>
				</TR>
				
				<TR BGCOLOR="#FFD700">
					<TD CLASS="Header">SKU</TD>
					<TD CLASS="Header" >Name</TD>
					<TD ALIGN="center" CLASS="Header">Qty</TD>
					<TD ALIGN="right" CLASS="Header">Price</TD>
					<TD ALIGN="right" CLASS="Header">Total</TD>
				</TR>
				<CFLOOP INDEX="x" FROM="1" TO="#ArrayLen(SESSION.aCart)#">
				<CFOUTPUT>
				<CFIF x MOD 2 IS 0>
					<!--- even row color --->
					<TR BGCOLOR="##E8E8E8">
		  		 <CFELSE>
		  	        <!--- odd row color --->
					<TR BGCOLOR="##FFFFFF">
				</CFIF>
					<TD VALIGN="top">#UCase(SESSION.aCart[x][1])#</TD>
					<TD>
						#SESSION.aCart[x][2]# <BR>
						<FONT SIZE="1"><A HREF="Cart.cfm?Action=Remove&Row=#x#">remove item</A></FONT>
					</TD>
					<TD ALIGN="center">#SESSION.aCart[x][3]#</TD>
					<TD ALIGN="right">#DollarFormat(SESSION.aCart[x][4])#</TD>
					<TD ALIGN="right">#DollarFormat(SESSION.aCart[x][3] * SESSION.aCart[x][4])#</TD> 
				</TR>
				</CFOUTPUT>
				<!--- calculate cart total --->
				<CFPARAM NAME="CartTotal" DEFAULT="0">
				<CFSET ItemTotal= SESSION.aCart[x][3] * SESSION.aCart[x][4]>
				<CFSET CartTotal=CartTotal + ItemTotal> 
				</CFLOOP>
				<TR>
					<TD>&nbsp;</TD>
					<TD>&nbsp;</TD>
					<TD>&nbsp;</TD>
					<TD><B>Cart Total</B></TD>
					<TD ALIGN="right"><CFOUTPUT><B>#DollarFormat(CartTotal)#</B></CFOUTPUT></TD> 
				</TR>
			</TABLE> 
			<!--- cart options --->
			<TABLE WIDTH="450" ALIGN="Center" CELLSPACING="0" CELLPADDING="3">
				<TR>
					<TD COLSPAN="4">
						<B>Cart Options</B>
					</TD>
				</TR>
				<TR>
					<TD><A HREF="##" onClick="self.close()">Continue To Shop</A></TD>
					<TD><A HREF="Cart.cfm?Action=Edit">Change Quantities</A></TD>
					<TD><A HREF="Cart.cfm?Action=Clear">Empty Cart</A></TD>
					<TD><A HREF="CartAction.cfm">Check Out</A></TD>
				</TR>
			</TABLE>
		   <CFELSE>
				<DIV ALIGN="center">
				<P>Your cart is currently empty. <BR>
				Please browse through our products 
				and add some items to to you cart.</P>
				<P><A HREF="##" onClick="self.close()">Continue To Shop</A></P>
				</DIV>
		  </CFIF>	
		  </CFLOCK>
		</CFCASE>
		
		<!--- add an item--->
		<CFCASE VALUE="Add">
			<!--- run query to get details of the item to be added --->
			<CFQUERY NAME="qProductDetail" DATASOURCE="#BBWebAppDSN#">
				SELECT      *
				FROM        Products 
				WHERE       ProductNo = '#URL.pid#'
			</CFQUERY>
			<!--- determine current length of the array and add new item to the end 
				  assume a quantity of one, it can be changed later--->
			<CFLOCK TIMEOUT="10" THROWONTIMEOUT="No" TYPE="EXCLUSIVE" SCOPE="SESSION">
				<CFSET LastRow=ArrayLen(SESSION.aCart)>	  
				<CFSET SESSION.aCart[LastRow+1][1]=qProductDetail.ProductNo>
				<CFSET SESSION.aCart[LastRow+1][2]=qProductDetail.Name>
				<CFSET SESSION.aCart[LastRow+1][3]="1">
				<!--- see if this product is on special and use special price --->
				<CFIF qProductDetail.OnSpecial IS NOT 0>
					<CFSET SESSION.aCart[LastRow+1][4]=qProductDetail.SpecialPrice>	
				  <CFELSE>
					<CFSET SESSION.aCart[LastRow+1][4]=qProductDetail.Price>	
				</CFIF>
			</CFLOCK>
			<CFLOCATION URL="Cart.cfm?Acion=Display" ADDTOKEN="No">
		</CFCASE>
		
		<!--- remove a selected item --->
		<CFCASE VALUE="Remove">
			<!--- remove row number passed via URL--->
			<CFLOCK TIMEOUT="10" THROWONTIMEOUT="No" TYPE="EXCLUSIVE" SCOPE="SESSION">
			  <CFSET RemoveRow=ArrayDeleteAt(SESSION.aCart,URL.Row)>
			</CFLOCK>
			<CFLOCATION URL="Cart.cfm?Action=Display" ADDTOKEN="No">
		</CFCASE>
		
		<!--- edit item quantities --->
		<CFCASE VALUE="Edit">
			<FORM ACTION="Cart.cfm?Action=Update" METHOD="POST">
			<TABLE WIDTH="450" BORDER="0" CELLSPACING="0" CELLPADDING="3" ALIGN="Center">
				<TR>
					<TD COLSPAN="5">
						Change the quantities of any items and click the Update button. 
					</TD>
				</TR>
				<TR BGCOLOR="#FFD700">
					<TD CLASS="Header">SKU</TD>
					<TD CLASS="Header" >Name</TD>
					<TD ALIGN="center" CLASS="Header">Qty</TD>
					<TD ALIGN="right" CLASS="Header">Price</TD>
					<TD ALIGN="right" CLASS="Header">Total</TD>
				</TR>
				<CFLOCK TIMEOUT="10" THROWONTIMEOUT="No" TYPE="READONLY" SCOPE="SESSION">
				  <CFLOOP INDEX="x" FROM="1" TO="#ArrayLen(SESSION.aCart)#">
					<CFOUTPUT>
					<CFIF x MOD 2 IS 0>
						<!--- even row color --->
						<TR BGCOLOR="##E8E8E8">
			  		 <CFELSE>
			  	        <!--- odd row color --->
						<TR BGCOLOR="##FFFFFF">
					</CFIF>
						
						<TD>#UCase(SESSION.aCart[x][1])#</TD>
						<TD>#SESSION.aCart[x][2]#</TD>
						<!--- put input box here, populate with current qty --->
						<TD ALIGN="center">
							<INPUT TYPE="text"
							       NAME="#x#"
							       VALUE="#SESSION.aCart[x][3]#"
							       SIZE="3"
							       MAXLENGTH="4">
						</TD>
						<TD ALIGN="right">#DollarFormat(SESSION.aCart[x][4])#</TD>
						<TD ALIGN="right">#DollarFormat(SESSION.aCart[x][3] * SESSION.aCart[x][4])#</TD> 
						
					</TR>
					
					</CFOUTPUT>
					</CFLOOP> 
				</CFLOCK>
				<TR>
					<TD>&nbsp;</TD>
					<TD>&nbsp;</TD>
					<TD>&nbsp;</TD>
					<TD><B>Update Total</B></TD>
					<TD ALIGN="right"><INPUT TYPE="submit" VALUE="Update"></TD> 
				</TR>
			</TABLE> 
			</FORM>	
		</CFCASE>
		
		<!--- use array insert at to update qtys --->	
		<CFCASE VALUE="Update">
			<!--- make sure all form fields contain a value of at least 1 --->
			<CFLOCK TIMEOUT="10" THROWONTIMEOUT="No" TYPE="READONLY" SCOPE="SESSION">
			  <CFLOOP INDEX="x" FROM="1" TO="#ArrayLen(SESSION.aCart)#">
				<!--- check for blank values and set to 1--->
				<CFIF Form[x] IS "">
					<CFSET Form[x]=1>
				</CFIF>
				<!--- check for values less than 1 and reset to 1 --->
				<CFIF Form[x] LT 1>
					<CFSET Form[x]=1>
				</CFIF>
				<!--- check for non number values and reset to 1 --->
				<CFIF NOT IsNumeric(Form[x])>
					<CFSET Form[x]=1>
				</CFIF>
			  </CFLOOP> 
			</CFLOCK>
			<CFLOCK TIMEOUT="10" THROWONTIMEOUT="No" TYPE="READONLY" SCOPE="SESSION">
			  <CFLOOP INDEX="x" FROM="1" TO="#ArrayLen(SESSION.aCart)#">
				<!--- update new value, round if decimal is present--->
				<CFSET SESSION.aCart[x][3]=#Round(FORM[x])#> 
			  </CFLOOP> 
			</CFLOCK>
			<!--- display new totals --->
			<CFLOCATION URL="Cart.cfm?Action=Display" ADDTOKEN="No">
		</CFCASE>
		
		<!--- empty the cart --->
		<CFCASE VALUE="Clear">
			<!--- clear the array --->
			<CFLOCK TIMEOUT="10" THROWONTIMEOUT="No" TYPE="EXCLUSIVE" SCOPE="SESSION">
			  <CFSET DumpCart=ArrayClear(SESSION.aCart)>
			</CFLOCK>
			<CFLOCATION URL="Cart.cfm?Action=Display" ADDTOKEN="No">
		</CFCASE>
		
	</CFSWITCH>
  </BODY>
</HTML>
